//==========================================
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) DIGITEO - 2009 - Allan CORNET
// 
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//==========================================
// Export Libraries of Scilab for GCC - MinGW compiler
//==========================================
function bOK = VCtoGCCLib()
  if ( detectpexports() & detectdlltool() ) then
    bOK = [];
    DirCur = pwd();
    
    bOK = [bOK , ExportAtlasLibrary('blasplus', WSCI+'\bin')];

    if with_module('scicos') then
      bOK = [bOK , Exportalibrary('scicos', WSCI+'\bin')];
      bOK = [bOK , Exportalibrary('scicos_f', WSCI+'\bin')];
      bOK = [bOK , Exportalibrary('scicos_blocks', WSCI+'\bin')];
      bOK = [bOK , Exportalibrary('scicos_blocks_f', WSCI+'\bin')];
    end

    bOK = [bOK , Exportalibrary('LibScilab', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('MALLOC', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('libf2c', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('intersci', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('output_stream', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('dynamic_link', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('dynamic_link_f', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('integer', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('optimization_f', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('libjvm', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('scilocalization', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('libintl', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('linpack_f', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('call_scilab', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('time', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('api_scilab', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('hashtable', WSCI+'\bin')];
    bOK = [bOK , Exportalibrary('scilab_windows', WSCI+'\bin')];    
    
		chdir(DirCur);
		
		if and(bOK == %T) then
		  bOK = %t;
		end
    
  else
    if ilib_verbose() > 0 then
      if ~detectpexports() then
        warning(_('pexports not found.'));
      end
      if ~detectdlltool() then
        warning(_('dlltool not found.'));
      end
    end
    bOK = %f;
  end
endfunction 
//==========================================
function  bOK = ExportAtlasLibrary(libraryname, pathlib)

  destPath = SCIHOME + filesep() + 'MinGWlib';
  mkdir(destPath);
  if chdir(pathlib) == %f then 
    bOK = %f;
    return
  end
  
  if ( ilib_verbose() <> 0 ) then
    mprintf('\nExports from '+libraryname+'.dll\n');
  end
  
  command = getMinGWInstallPath()+filesep()+'bin'+filesep()+'pexports '+libraryname+'.dll >'+TMPDIR+filesep()+libraryname+'.vcdef';
  ierr = unix(command);
  if ierr <> 0 then
    bOK = %F;	
    return;
  end

  if ( ilib_verbose() <> 0 ) then
    mprintf('Converting Library');
  end
 
  fw = mopen(TMPDIR+filesep()+libraryname+'.def',"wt");
  fr = mopen(TMPDIR+filesep()+libraryname+'.vcdef',"rt");
  
  if (meof(fr) == 0) then 
    line = mfscanf(1,fr,"%s");
    mfprintf(fw,"LIBRARY blasplus.dll\n");
  end

  if (meof(fr) == 0) then 
    line = mfscanf(1,fr,"%s");
    mfprintf(fw,"EXPORTS\n");
  end  

  if (meof(fr) == 0) then 
    line = mfscanf(1,fr,"%s");
  end  

  i = 1;
  while ( meof(fr) == 0)
    line = mfscanf(1,fr,"%s");
    if (line ~= []) then
      mfprintf(fw,"_%s_  %s @%d\n",line,line,i);
      i=i+1;
    end
  end

  mclose(fw);
  mclose(fr);
  
  fr = mopen(TMPDIR+filesep()+libraryname+'.def','rt');
  VCDEF = mgetl(fr);
  mclose(fr);
  
  find_dot = grep(VCDEF,'/\.$/','r');
  VCDEF(find_dot) = '';
  fw = mopen(TMPDIR+filesep()+libraryname+'.def','wt');
  mputl(VCDEF,fw);
  mclose(fw);

  if ( ilib_verbose() <> 0 ) then
    mprintf('\n');
    mprintf(gettext('Build lib%s.a (Please waiting).\n'),libraryname);
  end

  command = getMinGWInstallPath()+filesep()+'bin'+filesep()+'dlltool.exe -d""'+TMPDIR+filesep()+libraryname+'.def""'+' -l""'+destPath+filesep()+'lib'+libraryname+'.a""';
  ierr = unix(command);
  if ierr <> 0 then
    bOK = %F;	
    return;
  end

  if fileinfo(destPath+filesep()+'lib'+libraryname+'.a') <> [] then
    bOK = %T;
  else
    bOK = %F;	
  end
endfunction
//==========================================
function bOK = Exportalibrary(libraryname, pathlib)
  destPath = SCIHOME + filesep() + 'MinGWlib';
  mkdir(destPath);

  if chdir(pathlib) == %f then 
    bOK = %f;
    return
  end

  if ( ilib_verbose() <> 0 ) then  
    mprintf('\nExports from '+libraryname+'.dll\n');
  end
  command = getMinGWInstallPath()+filesep()+'bin'+filesep()+'pexports '+libraryname+'.dll >'+TMPDIR+filesep()+libraryname+'.vcdef';
  ierr = unix(command);
  if ierr <> 0 then
    bOK=%F;	
    return;
  end
  
  fr = mopen(TMPDIR+filesep()+libraryname+'.vcdef','rt');
  fw = mopen(TMPDIR+filesep()+libraryname+'.def','wt');
  VCDEF = mgetl(fr);
  find_dot = grep(VCDEF,'/\.$/','r');
  VCDEF(find_dot) = '';
  mputl(VCDEF,fw);
  mclose(fr);
  mclose(fw);

	if ( ilib_verbose() <> 0 ) then
    mprintf('Converting Library');
  end

  if ( ilib_verbose() <> 0 ) then
    mprintf('\n');
    mprintf(gettext('Build lib%s.a (Please waiting).\n'),libraryname);
  end
  command = getMinGWInstallPath()+filesep()+'bin'+filesep()+'dlltool.exe -d""'+TMPDIR+filesep()+libraryname+'.def""'+' -l""'+destPath+filesep()+'lib'+libraryname+'.a""';
  ierr = unix(command);
  if ierr <> 0 then
    bOK = %F;	
    return;
  end
  
  if fileinfo(destPath+filesep()+'lib'+libraryname+'.a') <> [] then
    bOK = %T;
  else
    bOK = %F;	
  end

endfunction
//==========================================
