// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) DIGITEO - 2009 - Allan CORNET
// 
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//==========================================
// Generate a shared library which can be used by link command. 
function libn = ilib_for_link(names, ..
                              files, ..
                              libs, ..
                              flag, ..
                              makename, ..
                              loadername, ..
                              libname, ..
                              ldflags, ..
                              cflags, ..
                              fflags, ..
                              cc)

  [lhs,rhs] = argn(0);
  
  if rhs <= 4 then makename = "Makelib";end
  if rhs <= 5 then loadername = "loader.sce";end
  if rhs <= 6 then libname = ""; end
  if rhs <= 7 then ldflags = ""; end 
  if rhs <= 8 then cflags  = ""; end 
  if rhs <= 9 then fflags  = ""; end 
  if rhs <= 10 then cc  = ""; end 
  
  // generate a loader file
  if ( ilib_verbose() <> 0 ) then
    mprintf(gettext("   Generate a loader file\n"));
  end
  ilib_link_gen_loader(names,flag,loadername,libs,libname);
  
  // generate a Makefile
  if ( ilib_verbose() <> 0 ) then
    mprintf(gettext("   Generate a Makefile\n"));
  end

  ilib_link_gen_Make(names, ..
                     files, ..
                     libs, ..
                     makename, ..
                     libname, ..
		                 ldflags, ..
		                 cflags, ..
		                 fflags, ..
		                 cc, ..
		                 flag);
		                 
  // we call make
  if ( ilib_verbose() <> 0 ) then
    mprintf(gettext("   Running the Makefile\n"));
  end
  
  if (libname == "") then libname = names(1);end
  libn = ilib_compile(libname, makename, files);
  
endfunction
//==========================================
function ilib_link_gen_Make(names, ..
                            files, ..
                            libs, ..
                            makename, ..
                            libname, ..
                            ldflags, ..
                            cflags, ..
                            fflags, ..
                            cc, ..
                            flag)

  // generate a Makefile for gateway
  [lhs,rhs] = argn(0);
  if rhs <= 2 then libs = [];end
  if rhs <= 3 then makename = 'Makelib';end
  if rhs <= 4 then libname = "";end
  if rhs <= 5 then ldflags = ""; end 
  if rhs <= 6 then cflags  = ""; end 
  if rhs <= 7 then fflags  = ""; end 
  if rhs <= 8 then cc  = ""; end 
  if rhs <= 9 then flag  = "c"; end 

  ilib_link_gen_Make_mingw(names, ..
                          files, ..
                          libs, ..
                          makename, ..
                          libname, ..
	                        ldflags, ..
		                      cflags, ..
		                      fflags, ..
		                      cc);

endfunction
//==========================================
function ilib_link_gen_loader(names,flag,loadername,libs,libname)

  rhs = argn(2);
  if rhs <= 4 then libname = ""; end 
  if rhs <= 3 then libs=[]; end 
  if rhs <= 2 then loadername = 'loader.sce' ; end 
  
  lib_suf = getdynlibext();
  if libname=="" then libname = names(1);end 
  
  if ( length(libname) + length('_path') ) > 24 then
   shortlibname_path = part(libname,1:(24 - length('_path')));
  else
   shortlibname_path = libname;
  end
  
  fd=mopen(loadername,"wt");
  mfprintf(fd,"// ------------------------------------------------------\n");
  mfprintf(fd,"// generated by builder.sce: Please do not edit this file\n");
  mfprintf(fd,"// ------------------------------------------------------\n");
  mfprintf(fd,"\n");
  mfprintf(fd,"%s_path = get_absolute_file_path(''%s'');\n",shortlibname_path,basename(loadername+'.x'));
  mfprintf(fd,"\n");
  
  //** first "link" : external libraries 
  if libs=="" then 
    //** do nothing : you don't need to link any "external" lib(s)
  else
    //** add one "link" line for each library (with the appropriate extension)
      nl = size(libs,'*') ;
      for i=1:nl 
         if ( part(libs(i),1) == '/') then
           mfprintf(fd,"link(''%s%s'');\n",libs(i),lib_suf);
         else
           [diri, basenamei, exti] = fileparts(libs(i));
           if (diri == '') then
            mfprintf(fd,"link(%s_path+''%s%s'');\n",shortlibname_path,libs(i),lib_suf);
           else
            mfprintf(fd,"link(''%s%s'');\n",libs(i),lib_suf);
           end
         end
      end
  end  
  
  //** second "link" : user defined functions  
  mfprintf(fd,"link(%s_path+''lib%s%s'',[",shortlibname_path,libname,lib_suf);
  
  names = names(:)';
  n = size(names,'*');
  for i=1:n
    mfprintf(fd,"''%s''",names(i))
    if i <>n ; mfprintf(fd,","); else mfprintf(fd,"],");end
  end
  
  mfprintf(fd,"''%s'');\n",flag);
  mfprintf(fd,"// remove temp. variables on stack\n");
  mfprintf(fd,"clear %s_path;\n",shortlibname_path);
  mfprintf(fd,"clear get_file_path;\n");
  mfprintf(fd,"// ------------------------------------------------------\n");
  mclose(fd);
  
  if ilib_verbose() > 1 then
    disp(mgetl(loadername));
  end
  
endfunction
//==========================================    
function ilib_link_gen_Make_mingw(names, ..
                                files, ..
                                libs, ..
                                Makename, ..
                                libname, ..
                                ldflags, ..
                                cflags, ..
                                fflags, ..
                                cc)
                                
                                
  managed_ext = ['.c', '.f'];
  obj_ext = ['.o', '.obj', ''];

  GCCLIBDIR = getMinGWInstallPath()+ filesep() + 'lib'
  SCIDIR = SCI;
  SCIDIR1= pathconvert(SCI,%f,%f,'w');
  SCIMINGWLIBDIR = SCIHOME + filesep() + 'MinGWlib';
  CFLAGS = cflags;
  MEXCFLAGS = '';
  FFLAGS = fflags;
  MEXFFLAGS = '';
  LDFLAGS = ldflags;
  LIBRARY = '';
  FILES_SRC = '';
  OBJS = '';
  OTHERLIBS = '';
  
  
  FILES_SRC_MATRIX = [];
  
  [path_Make, file_Make, ext_Make] = fileparts(Makename);
  
  for i=1:size(files,'*') 
    [path_f, file_f, ext_f] = fileparts(files(i));
    
    if or(obj_ext == ext_f) then
      FILENAME = [];
      FILE_FOUNDED = %f;
      for y = managed_ext(:)'
        if (FILE_FOUNDED == %f) then
          if (fileinfo(path_f + file_f + y) <> []) | (fileinfo(path_Make + file_f + y) <> []) then
            FILENAME = path_f + file_f + y;
            FILE_FOUNDED = %t;
          end
        end
      end
    else
      FILENAME = files(i);
    end
    FILES_SRC_MATRIX = [FILES_SRC_MATRIX , FILENAME];
  end
  
  FILES_SRC = strcat(FILES_SRC_MATRIX,' ');

  OBJS_MATRIX = [];
  
  for y = 1:size(FILES_SRC_MATRIX,'*')
    [path_f, file_f, ext_f] = fileparts(FILES_SRC_MATRIX(y));
    OBJS_MATRIX = [OBJS_MATRIX, path_f + file_f + '.o'];
  end
  
  OBJS = strcat(OBJS_MATRIX, ' ');
  
  if (libname == '') then
    if strncpy(names(1),3) <> 'lib' then
      LIBRARY = 'lib' + names(1);
    else
      LIBRARY = names(1);
    end
  else
    if strncpy(libname,3) <> 'lib' then
      LIBRARY = 'lib' + libname;
    else
      LIBRARY = libname;
    end
  end
  
  if (libs <> []) then
    for x = libs(:)'
      if x <> '' then
        OTHERLIBS = OTHERLIBS + ' ' + x + '.a';
      end
    end
  end
  OTHERLIBS = strsubst(OTHERLIBS,'/',filesep());

  path_script = '';
  try
   if isdef('mingwlib') then
     lib_str = string(mingwlib);
     path_script = lib_str(1) + '../scripts/';
   else
     path_script = '';
   end

   MAKEFILE_MINGW = mgetl(path_script + 'TEMPLATE_MAKEFILE.MINGW');
   
  catch
    MAKEFILE_MINGW = '';
    warning(path_script + 'TEMPLATE_MAKEFILE.MINGW (1) '+ _('not found.') );
    return
  end

  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__GCCLIBDIR__" , GCCLIBDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCI__" , SCIDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCIDIR1__" , SCIDIR1);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCIMINGWLIBDIR__" , SCIMINGWLIBDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__CFLAGS__" , CFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__MEXCFLAGS__" , MEXCFLAGS); 
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__FFLAGS__" , FFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__MEXFFLAGS__" , MEXFFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__LDFLAGS__" , LDFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__LIBNAME__" , LIBRARY);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__FILES_SRC__" , FILES_SRC);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__OBJS__" , OBJS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__OTHERSLIBS__" , OTHERLIBS);

  
  fd = mopen(Makename, "wt");
  mputl(MAKEFILE_MINGW, fd);
  mclose(fd);
    
  if ilib_verbose() > 1 then
    disp(mgetl(Makename));
  end
  
endfunction
//==========================================    
