// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) ENPC
// Copyright (C) DIGITEO - 2009 - Allan CORNET
// 
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function Makename=ilib_gen_Make(name,tables,files,libs,makename,with_gateway,ldflags,cflags,fflags,cc)
//------------------------------------
// generate a Makefile for gateway

  if argn(2)<6 then with_gateway=%t,ldflags='',cflags='',fflags='', cc='';end
  for i=1:size(files,'*') // compatibility scilab 4.x
    [path_f, file_f, ext_f] = fileparts(files(i));
    if or(ext_f == ['.o','.obj']) then
      files(i) = path_f + file_f;
    else
      files(i) = path_f + file_f + ext_f;
    end
  end
  
  // change table if necessary 
  if typeof(tables)<>'list' then 
    tables= list(tables)
  end
  L=length(tables); 
  for it = 1:L 
    table = tables(it);
    [mt,nt]=size(table);
	
	if nt==2 then 
	  col= "csci";  // @TODO what is csci
	  table = [table, col(ones(mt,1))];
	  nt=3; 
	end 
	
	if nt<>3 then 
	  error(msprintf(gettext("%s: Wrong size for input argument #%d.\n"),"ilib_gen_Make",2));
	end 
	
	tables(it)=table;
  end

  ilib_gen_Make_mingw(name,tables,files,libs,makename,with_gateway,ldflags,cflags,fflags)
  Makename = makename;
  
endfunction
//---------------------------------------------------------------------------------------
function ilib_gen_Make_mingw(name,table,files,libs,Makename,with_gateway,ldflags,cflags,fflags)
  managed_ext = ['.c', '.f'];
  obj_ext = ['.o', '.obj', ''];
  
  GCCLIBDIR = getMinGWInstallPath()+ filesep() + 'lib'
  SCIDIR = SCI;
  SCIDIR1= pathconvert(SCI,%f,%f,'w');
  SCIMINGWLIBDIR = SCIHOME + filesep() + 'MinGWlib';
  CFLAGS = cflags;
  MEXCFLAGS = '';
  FFLAGS = fflags;
  MEXFFLAGS = '';
  LDFLAGS = ldflags;
  LIBRARY = name;
  FILES_SRC = '';
  OBJS = '';
  OTHERLIBS = '';

  
  FILES_SRC_MATRIX = [];
  
  [path_Make, file_Make, ext_Make] = fileparts(Makename);
  
  for i=1:size(files,'*') 
    [path_f, file_f, ext_f] = fileparts(files(i));
    
    if or(obj_ext == ext_f) then
      FILENAME = [];
      FILE_FOUNDED = %f;
      for y = managed_ext(:)'
        if (FILE_FOUNDED == %f) then
          if (fileinfo(path_f + file_f + y) <> []) | (fileinfo(path_Make + file_f + y) <> []) then
            FILENAME = path_f + file_f + y;
            FILE_FOUNDED = %t;
          end
        end
      end
    else
      FILENAME = files(i);
    end
    FILES_SRC_MATRIX = [FILES_SRC_MATRIX , FILENAME];
  end

  if typeof(tables) <> 'list' then 
    tables = list(tables);
  end
  L = length(tables); 

  if with_gateway then 
    if L == 1 then 
      FILES_SRC_MATRIX = [FILES_SRC_MATRIX , name + '.c'];
    else
       for i=1:L
        FILES_SRC_MATRIX = [FILES_SRC_MATRIX , name + string(i) + '.c'];
       end 
    end
  end
  
  for it=1:L 
    table = tables(it);
    [mt,nt] = size(table);
 
    for i=1:mt ; 
      // mex files to be added 
      if table(i,3)=='cmex' | table(i,3)=='fmex' | table(i,3)=='Fmex' 
        FILES_SRC_MATRIX = [FILES_SRC_MATRIX , table(i,2)];
      end
    end
  end
  
  FILES_SRC = strcat(FILES_SRC_MATRIX,' ');
  
  if table(i,3)=='cmex' | table(i,3)=='fmex' | table(i,3)=='Fmex' then
    MEXCFLAGS = "-Dmexfunction_=mex$*_ -DmexFunction=mex_$*";
    MEXFFLAGS = "-Dmexfunction=mex$*";
  end 

  OBJS_MATRIX = [];
  
  for y = 1:size(FILES_SRC_MATRIX,'*')
    [path_f, file_f, ext_f] = fileparts(FILES_SRC_MATRIX(y));
    OBJS_MATRIX = [OBJS_MATRIX, path_f + file_f + '.o'];
  end
  
  OBJS = strcat(OBJS_MATRIX, ' ');
  

  if strncpy(LIBRARY,3) <> 'lib' then
    LIBRARY = 'lib' + LIBRARY;
  end

  for x=libs(:)'
     if OTHERLIBS <> '' then
       OTHERLIBS = OTHERLIBS + ' ' + x + '.a';
     else
       OTHERLIBS = x + '.a';
     end 
  end
  OTHERLIBS = strsubst(OTHERLIBS,'/',filesep());

  path_script = '';
  try
   if isdef('mingwlib') then
     lib_str = string(mingwlib);
     path_script = lib_str(1) + '../scripts/';
   else
     path_script = '';
   end

   MAKEFILE_MINGW = mgetl(path_script + 'TEMPLATE_MAKEFILE.MINGW');
   
  catch
    MAKEFILE_MINGW = '';
    warning(path_script + 'TEMPLATE_MAKEFILE.MINGW (1) '+ _('not found.') );
    return
  end

  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__GCCLIBDIR__" , GCCLIBDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCI__" , SCIDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCIDIR1__" , SCIDIR1);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__SCIMINGWLIBDIR__" , SCIMINGWLIBDIR);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__CFLAGS__" , CFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__MEXCFLAGS__" , MEXCFLAGS); 
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__FFLAGS__" , FFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__MEXFFLAGS__" , MEXFFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__LDFLAGS__" , LDFLAGS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__LIBNAME__" , LIBRARY);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__FILES_SRC__" , FILES_SRC);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__OBJS__" , OBJS);
  MAKEFILE_MINGW = strsubst(MAKEFILE_MINGW , "__OTHERSLIBS__" , OTHERLIBS);

  
  fd = mopen(Makename, "wt");
  mputl(MAKEFILE_MINGW, fd);
  mclose(fd);
    
  if ilib_verbose() > 1 then
    disp(mgetl(Makename));
  end
  
endfunction
//---------------------------------------------------------------------------------------
